@@ -4,68 +4,51 @@ import ast
 
 
 class Where(ast.AST):
-    def __init__(self, source, predicate):
-        self._fields = ['source', 'predicate']
-        self.source = source
-        self.predicate = predicate
+    _fields = ['source', 'predicate']
 
 
 class Select(ast.AST):
-    def __init__(self, source, selector):
-        self._fields = ['source', 'selector']
-        self.source = source
-        self.selector = selector
+    _fields = ['source', 'selector']
 
 
 class SelectMany(ast.AST):
-    def __init__(self, source, selector):
-        self._fields = ['source', 'selector']
-        self.source = source
-        self.selector = selector
+    _fields = ['source', 'selector']
 
 
 class First(ast.AST):
-    def __init__(self, source):
-        self._fields = ['source']
-        self.source = source
+    _fields = ['source']
 
 
 class Aggregate(ast.AST):
-    def __init__(self, source, seed, func):
-        self._fields = ['source', 'seed', 'func']
-        self.source = source
-        self.seed = seed
-        self.func = func
+    _fields = ['source', 'seed', 'func']
 
 
 class Count(ast.AST):
-    def __init__(self, source):
-        self._fields = ['source']
-        self.source = source
+    _fields = ['source']
 
 
 class Max(ast.AST):
-    def __init__(self, source):
-        self._fields = ['source']
-        self.source = source
+    _fields = ['source']
 
 
 class Min(ast.AST):
-    def __init__(self, source):
-        self._fields = ['source']
-        self.source = source
+    _fields = ['source']
 
 
 class Sum(ast.AST):
-    def __init__(self, source):
-        self._fields = ['source']
-        self.source = source
+    _fields = ['source']
 
 
 class Zip(ast.AST):
-    def __init__(self, source):
-        self._fields = ['source']
-        self.source = source
+    _fields = ['source']
+
+
+class OrderBy(ast.AST):
+    _fields = ['source', 'key_selector']
+
+
+class Choose(ast.AST):
+    _fields = ['source', 'n']
 
 
 linq_operator_names = ('Where',
@@ -77,7 +60,9 @@ linq_operator_names = ('Where',
                        'Max',
                        'Min',
                        'Sum',
-                       'Zip')
+                       'Zip',
+                       'OrderBy',
+                       'Choose')
 
 
 class InsertLINQNodesTransformer(ast.NodeTransformer):
@@ -161,6 +146,19 @@ class InsertLINQNodesTransformer(ast.NodeTransformer):
             if len(args) != 0:
                 raise SyntaxError('Zip() call must have zero arguments')
             return Zip(source=self.visit(source))
+        elif function_name == 'OrderBy':
+            if len(args) != 1:
+                raise SyntaxError('OrderBy() call must have exactly one argument')
+            if isinstance(args[0], ast.Str):
+                args[0] = unwrap_ast(ast.parse(args[0].s))
+            if not isinstance(args[0], ast.Lambda):
+                raise SyntaxError('OrderBy() call argument must be a lambda')
+            return OrderBy(source=self.visit(source),
+                           key_selector=self.visit(args[0]))
+        elif function_name == 'Choose':
+            if len(args) != 1:
+                raise SyntaxError('Choose() call must have exactly one argument')
+            return Choose(source=self.visit(source), n=self.visit(args[0]))
         else:
             raise NameError('Unhandled LINQ operator: ' + function_name)
 
